#!/usr/bin/env python3

from sys import platform
from argparse import ArgumentParser
from contextlib import redirect_stdout
from os import devnull
from sys import stdout, stderr, exit
from pathlib import Path
from woven import WovenConfig
from json import JSONDecodeError
from cattrs.errors import ClassValidationError
import customtkinter as ctk

def start(config_path: str):
    ctk.set_appearance_mode("System")
    ctk.set_default_color_theme("dark-blue")

    if platform != "darwin" and not platform.startswith("win"):
        ctk.set_widget_scaling(1.25)
        ctk.set_window_scaling(1.25)

    app = ctk.CTk()
    app.geometry("1200x800")
    app.title("Woven")

    config = None
    try:
        config = WovenConfig.load_json_file(config_path)
    except FileNotFoundError:
        print(f"No file found at '{config_path}'")
    except JSONDecodeError as e:
        print(f"Invalid JSON encountered in configuration file: {e}")
    except ClassValidationError as e:
        print(f"The following validation errors occurred when loading the configuration file:", file = stderr)
        for e in e.exceptions:
            print(f"{type(e).__name__}: {e}", file = stderr)
    
    if config is not None:
        config_textbox = ctk.CTkTextbox(master = app)
        config_textbox.insert("0.0", config.to_json_str())
        config_textbox.place(relx = 0.5, rely = 0.5, anchor = ctk.CENTER)
    else:
        button = ctk.CTkButton(master = app, text = "Load config", command = lambda: print("button pressed"))
        button.place(relx = 0.5, rely = 0.5, anchor = ctk.CENTER)

    app.mainloop()

def main():
        parser = ArgumentParser("woven-ui")
        parser.add_argument("-q", "--quiet", action = "store_true", help = "decrease output verbosity")
        parser.add_argument("-c", "--config", default = "config.json", help = "The path to the config file")
        args = parser.parse_args()
        
        with redirect_stdout(open(devnull, "w") if args.quiet else stdout):
            start(args.config)

if __name__ == "__main__":
    main()
